@@ -1,7 +1,13 @@
+#![allow(unused_imports)]
+
+#[cfg(test)]
+pub(crate) use crate::add_port;
+
 pub(crate) use crate::client;
 pub(crate) use crate::config;
 pub(crate) use crate::config::requirements::{NodeProvidedCaps, Requirements};
 pub(crate) use crate::error;
+pub(crate) use crate::protocol;
 pub(crate) use crate::server;
 pub(crate) use crate::transport;
 
@@ -9,6 +15,7 @@ pub(crate) use std::path::Path;
 pub(crate) use std::path::PathBuf;
 pub(crate) use tokio::io::{AsyncRead, AsyncWrite};
 pub(crate) use tokio::io::{AsyncReadExt, AsyncWriteExt};
+pub(crate) use tokio::net::{TcpListener, TcpStream};
 pub(crate) use tokio::sync::broadcast;
 pub(crate) use tokio::sync::mpsc;
 pub(crate) use tokio::sync::oneshot;
