@@ -79,7 +79,7 @@ async fn check_pull() {
     dbg!(&server);
 
     // start the server
-    tokio::task::spawn_local(async move {
+    tokio::task::spawn(async move {
         println!("starting server");
         distribute::server_command(server).await.unwrap();
         println!("server has exited");
@@ -180,7 +180,7 @@ async fn pull_large_file() {
     dbg!(&server);
 
     // start the server
-    tokio::task::spawn_local(async move {
+    tokio::task::spawn(async move {
         println!("starting server");
         distribute::server_command(server).await.unwrap();
         println!("server has exited");
